import cv2
import numpy as np
from cozmo_fsm import *

#Runs the contour program on the given camera image and shape matches it to house.jpg
class ShapeMatch(StateMachineProgram):


    def __init__(self):
        super().__init__(aruco=False, particle_filter=False, cam_viewer=False,
                         annotate_sdk=False)
        
    #Starts the program and the 2 display windows
    def start(self):
        dummy = numpy.array([[0]], dtype='uint8')
        super().start()

        cv2.namedWindow('camera image')
        cv2.imshow('camera image',dummy)

    #Copied from match_shapes.py with small adjustments
    def do_contours(self, image):
        colors = [(0,0,255), (0,255,0), (255,0,0),
                  (255,255,0), (255,0,255), (0,255,255),
                  (0,0,128), (0,128,0), (128,0,0),
                  (128,128,0), (0,128,128), (128,0,128),
                  (255,255,255)]
        image_gray = cv2.cvtColor(image, cv2.COLOR_RGB2GRAY)
        ret, image_thresh = cv2.threshold(image_gray, 127, 255, 0)
        contours, hierarchy = \
                cv2.findContours(image_thresh, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
        areas = [(i, cv2.contourArea(contours[i])) for i in range(len(contours))]
        areas.sort(key=lambda x: x[1])
        areas.reverse()
        #We increased the min area because there is too much camera noise
        minArea = 100
        result = []
        for area_entry in areas:
            if area_entry[1] < minArea:
                break
            temp = index = area_entry[0]
            depth = -1
            while temp != -1 and depth < len(colors)-1:
                depth += 1
                temp = hierarchy[0,temp,3]
            contour = contours[index]
            cv2.drawContours(image, [contour], 0, colors[depth], 1)
            result.append(contour)
        return result

    #Adds contour lines to the current image and compares contour between
    #house.jpg and camera image
    def user_image(self, image, gray):
        
        house_im = cv2.imread('house.jpg')
        res_a = self.do_contours(house_im)        
        res_b = self.do_contours(image)
        rez = cv2.matchShapes(res_a[0], res_b[0], 1, 0.0)
        
        print(rez)
        #Shows the camera image and displays the match factor
        cv2.imshow('camera image', image)
        cv2.waitKey(1)

